/*
 * THERMOSTAT WITH LCD DISPLAY
 *
 * This program measures temperature with digital sensor DS1820,
 * and activates relay at a given temperature.
 *
 * PIC16F628
 * 4 MHz , RC clock
 *
 * Author: Ar2r, october 2010
 * Copyright  2010 Ar2r. All rights reserved.
 */
//------------------------------------------------------------------------------

#include "Ar2r.h"
#include "built_in.h"

unsigned short but1, but2, but3, tol, ttol, pwr, set, tick, bep;
short tem, stem, tstem;
unsigned temp;
unsigned char ch;
unsigned short ch1, ch2;
//------------------------------------------------------------------------------

// Reads 2 byte from digital sensor DS18B20
void ReadTemperature()
{
    Ow_Reset(&PORTA,2);                          // Onewire reset signal
    Ow_Write(&PORTA,2,0xCC);                     // Issue command SKIP_ROM
    Ow_Write(&PORTA,2,0x44);                     // Issue command CONVERT_T
    Delay_us(120);

    Ow_Reset(&PORTA,2);
    Ow_Write(&PORTA,2,0xCC);                     // Issue command SKIP_ROM
    Ow_Write(&PORTA,2,0xBE);                     // Issue command READ_SCRATCHPAD
    Delay_ms(300);

    Lo(temp) = Ow_Read(&PORTA,2);
    Hi(temp) = Ow_Read(&PORTA,2);
}
//------------------------------------------------------------------------------

void ChE(void)
{
    if(ch1==1)
    {
        EEprom_Write(0, stem);
        ch1=0;
    }
    if(ch2==1)
    {
        EEprom_Write(1, tol);
        ch2=0;
    }
}
//------------------------------------------------------------------------------

void LC1(void)
{
    ch=(tem/10) %10;                                // extract 10.00 U digit
    LCD_Chr(1,1, 32);
    if (ch==0)
    {
        LCD_Chr_Cp(32);                             // write empty space if digit is 0
    }
    else
    {
        LCD_Chr_Cp(48+ch);                          // write ASCII digit at 1st row, 2nd column
    }
    ch=tem %10;
    LCD_Chr_Cp(48+ch);
    Lcd_Chr_Cp(223);
    Lcd_Chr_Cp('C');
    Lcd_Chr_Cp(' ');
    Lcd_Chr_Cp(' ');
    Lcd_Chr_Cp(' ');
    
    ch=(stem/10) %10;                               // extract 10.00 U digit
    if (ch==0)
    {
        LCD_Chr(2,1, 32);                           // write empty space if digit is 0
    }
    else
    {
        LCD_Chr(2,1,48+ch);                         // write ASCII digit at 1st row, 2nd column
    }
    ch=stem %10;
    LCD_Chr_Cp(48+ch);
    Lcd_Chr_Cp(223);
    Lcd_Chr_Cp('C');
    Lcd_Chr_Cp(' ');
    switch(pwr)
    {
        case 0:
            LCD_Chr_Cp('O');
            Lcd_Chr_Cp('f');
            Lcd_Chr_Cp('f');
        break;
        case 1:
            LCD_Chr_Cp('O');
            Lcd_Chr_Cp('n');
            Lcd_Chr_Cp(' ');
        break;
    }
}
//------------------------------------------------------------------------------

void LC2(void)
{
    LCD_Out(1,1, "  Temp.=");
    ch=(stem/10) %10;
    if (ch==0)
    {
        LCD_Chr(2,1, 32);                             // write empty space if digit is 0
    }
    else
    {
        LCD_Chr(2,1, 48+ch);                          // write ASCII digit at 1st row, 2nd column
    }
    ch=stem %10;
    LCD_Chr_Cp(48+ch);
    Lcd_Chr_Cp(223);
    Lcd_Chr_Cp('C');
    Lcd_Chr_Cp(' ');
    Lcd_Chr_Cp(' ');
    Lcd_Chr_Cp(' ');
    Lcd_Chr_Cp(' ');
}
//------------------------------------------------------------------------------

void LC3(void)
{
    LCD_Out(1,1, "  Hyst.=");
    ch=(tol/10) %10;
    if (ch==0)
    {
        LCD_Chr(2,1, 32);                             // write empty space if digit is 0
    }
    else
    {
        LCD_Chr(2,1, 48+ch);                          // write ASCII digit at 1st row, 2nd column
    }
    ch=tol %10;
    LCD_Chr_Cp(48+ch);
    Lcd_Chr_Cp(223);
    Lcd_Chr_Cp('C');
    Lcd_Chr_Cp(' ');
    Lcd_Chr_Cp(' ');
    Lcd_Chr_Cp(' ');
    Lcd_Chr_Cp(' ');
}
//------------------------------------------------------------------------------

void Beep(int y, int c)
{
    int z;
    for(z=0;z<=c;z++)
    {
        PORTB.F6=1;
        Vdelay_ms(y);
        PORTB.F6=0;
        Vdelay_ms(y);
    }
}
//------------------------------------------------------------------------------

void main()
{
    CMCON      = 0b00000111;                        // Set digital input mode for PORTA
    TRISA      = 0b11111100;                        // Configure PORTA (0 is output, 1 is input)
    PORTA.F1   = 0;
    ch2=ch1=0; but1=but2=but3=0; pwr=0; bep=tick=set=0;
    //OPTION_REG  = 0b11000101;                     // no prescaler
    //INTCON      = 0b11100000;                     // Enable GIE, T0IE , PIE
    Lcd_Config(&PORTB, 0, 1, 6, 5, 4, 3, 2);        // parameters RS, EN, WR, D7 .. D4
    TRISB.F6    =    0;                             // Set PORTB D6 as output (buzzer)
    TRISB.F7    =    1;                             // Set PORTB D7 as input

    stem=Eeprom_Read(0);                            // Read value from eeprom
    if (stem<=0 || stem>=91)
    {
        stem=65;
        ch1=1;
    }
    tol=Eeprom_Read(1);                             // Read value from eeprom
    if (tol>=86)
    {
        tol=5;
        ch2=1;
    }

    Lcd_Cmd(Lcd_CURSOR_OFF);
    Ar2rSplash();
    LCD_Cmd(LCD_CLEAR);                             // send command  to LCD (clear LCD)

    while (1)
    {
        ChE();
        ReadTemperature();
        tem=ConTemp(temp);                          // Format result
        if(tem>=1 && tem<=stem-tol)                 // Activate relay
        {
            PORTA.F1=1;
            pwr=1;
            bep=1;
        }
        if(tem>stem || tem<=0)                      // Deactivate relay
        {
            PORTA.F1=0;
            pwr=0;
            if(bep==1)
            {
                Beep(1,1000);
                bep=0;
            }
        }
        if(tem<=0) Beep(2,200);
            
        switch(set)                                 // Display set modes
        {
            case 0: LC1(); break;
            case 1: LC2(); break;
            case 2: LC3(); break;
        }
            
        if(set>=1) tick++;
        if(tick>=40)
        {
            tick=set=0;
            tol=ttol;
            stem=tstem;
        }

        if (PORTA.F5==0) but3 = 1;                   // If up button is pressed
        if (but3==1 && PORTA.F5==1)
        {
            tick=but3 = 0;
            switch(set)
            {
                case 1:
                    stem++;
                    if(stem>=91) stem=1;
                    LC2();
                break;
                case 2:
                    tol++;
                    if(tol>=86) tol=1;
                    LC3();
                break;
            }
        }

        if (PORTA.F4==0) but2 = 1;                   // If down button is pressed
        if (but2==1 && PORTA.F4==1)
        {
            tick=but2 = 0;
            switch(set)
            {
                case 1:
                    stem--;
                    if(stem<=0) stem=90;
                    LC2();
                break;
                case 2:
                    tol--;
                    if(tol<=0) tol=85;
                    LC3();
                break;
            }
        }

        if (PORTA.F3==0) but1 = 1;                   // If set button is pressed
        if (but1==1 && PORTA.F3==1)
        {
            tick=but1 = 0;
            set++;
            if(set==1)
            {
                tstem=stem;
                ttol=tol;
            }
            if(set>=3)
            {
                tick=set=0;
                if(tstem!=stem) ch1=1;
                if(ttol!=tol) ch2=1;
            }
        }
    }
}
//------------------------------------------------------------------------------
